==> django-secretary_0.2.0.orig.tar.gz: django-secretary-0.2.0/django_secretary/utils/templates.py <==
"""Template utilities for django-secretary."""
from contextlib import ContextDecorator
from threading import local
from types import TracebackType
from typing import Any, Dict, Iterator, Optional, Type, cast

from django.template import EngineHandler, Origin, Template as TemplateObject, TemplateDoesNotExist
from django.template.backends.django import DjangoTemplates, Template as EngineTemplate  # type: ignore[attr-defined]
from django.template.context import make_context
from django.template.loaders.base import Loader
from django.utils.functional import cached_property

from ..models import Template as TemplateModel
from ..settings import SECRETARY_SETTINGS

TEMPLATE_ENGINES = EngineHandler(SECRETARY_SETTINGS.TEMPLATES)

# Thread-safe storage of active templates.
# If the attribute `template` exists, it contains the actual active template.
# If the attribute `templates` exists, it contains mapping of template name to `Template` model instance.
_ACTIVE_TEMPLATES = local()


def get_active_template(name: str = None) -> TemplateModel:
    """Return active template or raise an exception.

    Arguments:
        name: If provided, return one of the active templates matching the name.
              If missing, return the active template itself.

    Raises:
        TemplateDoesNotExist: If template was not found.
    """
    if name is None:
        if hasattr(_ACTIVE_TEMPLATES, 'template'):
            return cast(TemplateModel, _ACTIVE_TEMPLATES.template)
        else:
            raise TemplateDoesNotExist("There is no active template.")

    if hasattr(_ACTIVE_TEMPLATES, 'templates'):
        if name in _ACTIVE_TEMPLATES.templates:
            return cast(TemplateModel, _ACTIVE_TEMPLATES.templates[name])
        raise TemplateDoesNotExist("'{}' has no active template.".format(name))
    try:
        return TemplateModel.objects.get(name=name, is_active=True)
    except TemplateModel.DoesNotExist:
        raise TemplateDoesNotExist("'{}' is neither active, nor found in database.".format(name))


class override_template(ContextDecorator):
    """Context manager/decorator to force active template."""

    def __init__(self, template: TemplateModel):
        self.template = template
        self._old_template: Optional[TemplateModel] = None
        self._old_templates: Optional[Dict[str, TemplateModel]] = None

    def __enter__(self) -> None:
        self._old_template = getattr(_ACTIVE_TEMPLATES, 'template', None)
        self._old_templates = getattr(_ACTIVE_TEMPLATES, 'templates', None)

        active_templates = {}
        template: Optional[TemplateModel] = self.template
        while template is not None:
            active_templates[template.name] = template
            template = template.parent
        _ACTIVE_TEMPLATES.template = self.template
        _ACTIVE_TEMPLATES.templates = active_templates

    def __exit__(self, exc_type: Optional[Type[BaseException]], exc_value: Optional[BaseException],
                 traceback: Optional[TracebackType]) -> None:
        if self._old_template is None:
            del _ACTIVE_TEMPLATES.template
            del _ACTIVE_TEMPLATES.templates
        else:
            _ACTIVE_TEMPLATES.template = self._old_template
            _ACTIVE_TEMPLATES.templates = self._old_templates


class TemplateLoader(Loader):
    """Template loader which returns active templates from `Template` model."""

    def get_template_sources(self, template_name: str) -> Iterator[Origin]:
        """Return template origin."""
        yield Origin(name=template_name, template_name=template_name, loader=self)

    def get_contents(self, origin: Origin) -> str:
        """Return template contents or raise an exception.

        Raises:
            TemplateDoesNotExist: If template was not found.
        """
        template_file = get_active_template(origin.name).content
        try:
            return cast(bytes, template_file.read()).decode(encoding=self.engine.file_charset)
        finally:
            template_file.close()


class Template:
    """Object holding all data to render a secretary template.

    Has API similar to the engine template object, such as django.template.backends.django.Template.
    """

    def __init__(self, model: TemplateModel, backend: DjangoTemplates):
        self.model = model
        self.backend = backend

    @cached_property
    def template(self) -> TemplateObject:
        """Return a compiled Template object for this template."""
        with override_template(self.model):
            return cast(TemplateObject, cast(EngineTemplate, self.backend.get_template(self.model.name)).template)

    @property
    def origin(self) -> Origin:
        """Return a compiled Template origin."""
        return self.template.origin

    def render(self, context: Dict[str, Any]) -> str:
        """Render the template with provided context.

        Arguments:
            context: Context data to be rendered.
        """
        # Make a shallow copy to prevent unwated changes in provided context.
        context = context.copy()
        context['template'] = self.model
        context_object = make_context(context, autoescape=self.backend.engine.autoescape)
        with override_template(self.model):
            return cast(str, self.template.render(context_object))

==> django-secretary_0.5.2.orig.tar.gz: django-secretary-0.5.2/django_secretary/utils/templates.py <==
"""Template utilities for django-secretary."""
from contextlib import ContextDecorator
from threading import local
from types import TracebackType
from typing import Any, Dict, Iterator, Optional, Type, cast

from django.template import EngineHandler, Origin, Template as TemplateObject, TemplateDoesNotExist
from django.template.backends.django import DjangoTemplates, Template as EngineTemplate  # type: ignore[attr-defined]
from django.template.context import make_context
from django.template.loaders.base import Loader
from django.utils.functional import cached_property

from ..models import Template as TemplateModel
from ..settings import SECRETARY_SETTINGS

TEMPLATE_ENGINES = EngineHandler(SECRETARY_SETTINGS.TEMPLATES)

# Thread-safe storage of active templates.
# If the attribute `template` exists, it contains the actual active template.
# If the attribute `templates` exists, it contains mapping of template name to `Template` model instance.
_ACTIVE_TEMPLATES = local()


def get_active_template(name: Optional[str] = None) -> TemplateModel:
    """Return active template or raise an exception.

    Arguments:
        name: If provided, return one of the active templates matching the name.
              If missing, return the active template itself.

    Raises:
        TemplateDoesNotExist: If template was not found.
    """
    if name is None:
        if hasattr(_ACTIVE_TEMPLATES, 'template'):
            return cast(TemplateModel, _ACTIVE_TEMPLATES.template)
        else:
            raise TemplateDoesNotExist("There is no active template.")

    if hasattr(_ACTIVE_TEMPLATES, 'templates'):
        if name in _ACTIVE_TEMPLATES.templates:
            return cast(TemplateModel, _ACTIVE_TEMPLATES.templates[name])
        raise TemplateDoesNotExist("'{}' has no active template.".format(name))
    try:
        return TemplateModel.objects.get(name=name, is_active=True)
    except TemplateModel.DoesNotExist:
        raise TemplateDoesNotExist("'{}' is neither active, nor found in database.".format(name))


class override_template(ContextDecorator):
    """Context manager/decorator to force active template."""

    def __init__(self, template: TemplateModel):
        self.template = template
        self._old_template: Optional[TemplateModel] = None
        self._old_templates: Optional[Dict[str, TemplateModel]] = None

    def __enter__(self) -> None:
        self._old_template = getattr(_ACTIVE_TEMPLATES, 'template', None)
        self._old_templates = getattr(_ACTIVE_TEMPLATES, 'templates', None)

        active_templates = {}
        template: Optional[TemplateModel] = self.template
        while template is not None:
            active_templates[template.name] = template
            template = template.parent
        _ACTIVE_TEMPLATES.template = self.template
        _ACTIVE_TEMPLATES.templates = active_templates

    def __exit__(self, exc_type: Optional[Type[BaseException]], exc_value: Optional[BaseException],
                 traceback: Optional[TracebackType]) -> None:
        if self._old_template is None:
            del _ACTIVE_TEMPLATES.template
            del _ACTIVE_TEMPLATES.templates
        else:
            _ACTIVE_TEMPLATES.template = self._old_template
            _ACTIVE_TEMPLATES.templates = self._old_templates


class TemplateLoader(Loader):
    """Template loader which returns active templates from `Template` model."""

    def get_template_sources(self, template_name: str) -> Iterator[Origin]:
        """Return template origin."""
        yield Origin(name=template_name, template_name=template_name, loader=self)

    def get_contents(self, origin: Origin) -> str:
        """Return template contents or raise an exception.

        Raises:
            TemplateDoesNotExist: If template was not found.
        """
        template_file = get_active_template(origin.name).content
        try:
            return cast(bytes, template_file.read()).decode(encoding=self.engine.file_charset)
        finally:
            template_file.close()


class Template:
    """Object holding all data to render a secretary template.

    Has API similar to the engine template object, such as django.template.backends.django.Template.
    """

    def __init__(self, model: TemplateModel, backend: DjangoTemplates):
        self.model = model
        self.backend = backend

    @cached_property
    def template(self) -> TemplateObject:
        """Return a compiled Template object for this template."""
        with override_template(self.model):
            return cast(TemplateObject, cast(EngineTemplate, self.backend.get_template(self.model.name)).template)

    @property
    def origin(self) -> Origin:
        """Return a compiled Template origin."""
        return self.template.origin

    def render(self, context: Dict[str, Any]) -> str:
        """Render the template with provided context.

        Arguments:
            context: Context data to be rendered.
        """
        # Make a shallow copy to prevent unwated changes in provided context.
        context = context.copy()
        context['template'] = self.model
        context_object = make_context(context, autoescape=self.backend.engine.autoescape)
        with override_template(self.model):
            return cast(str, self.template.render(context_object))
